//
//  SimpleViewController.m
//  SimpleUSBeaconApp
//
//  Created by wezger on 2014/5/16.
//  Copyright (c) 2014年 THLight Ltd. All rights reserved.
//

#import "SimpleViewController.h"

#define TEST_POWER_UUID @"00112233-4455-6677-8899-AABBCCDDEEFF"

@interface SimpleViewController ()

@property (strong,nonatomic) USBCManager * manager;
@property (retain,nonatomic) BeaconDetect * detect;

//UI
@property (retain,nonatomic) IBOutlet UILabel * nearistBeaconLabel;
@property (retain,nonatomic) IBOutlet UITextView * beaconDataTextView;
@property (retain,nonatomic) IBOutlet UILabel * BeaconPowerLabel;

@end

@implementation SimpleViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //USBC Data Fetch.  ( To keep it simple, we just fetch data from usbeacom.com.tw every time the App start.)
    _manager = [USBCManager defaultManager];
    _manager.delegate = self;
    /********** REPLACE the Data Query UUID wit yours *********/
    [_manager updateDevicesWithDataQueryUUID:@"61552DF0-D0B7-4497-8DD0-2B69DF007B13"]; //It's a Data Query UUID of test@thlight.com
}


#pragma mark USBCManagerDelegate

-(void)USBCManagerUpdateComplete
{
    
    NSLog(@"In %s Beacon Data Fetched from Server has been saved in your iPhone/iPad", __func__);
    
    //Print All Fetched Data
    
    NSLog(@"List all USBeacon device information");
    NSArray * array = [_manager allDevices];
    for (int i = 0; i < array.count; i++) {
        USBeaconDevice * device = (USBeaconDevice *)array[i];
        [device logDevice];
    }
    
    //Let our iPhone search nearby iBeacons.
    NSArray * uuids = @[@"E2C56DB5-DFFB-48D2-B060-D0F5A71096E0",@"23542266-18D1-4FE4-B4A1-23F8195B9D39"];
    //NSArray * uuids = @[@"D3CF0C2B-0D8F-447B-87B5-ABA77D5A967E"];
    
    _detect = [BeaconDetect detectBeaconWithUUIDs:uuids];
    _detect.delegate = self;
    
    //set the detect power voltage special UUID with yours
    [_detect setUUIDForDetectPowerVoltage:TEST_POWER_UUID];
    
    [_detect startSearching];
}

-(void)USBCManagerUpdateError:(NSError *)error
{
    //NSLog(@"%@",error);
}


#pragma mark BeaconDetectDelegate
-(void)beaconListChangeTo:(NSArray*)beacons
{
    //Update Beacon List Here(including all UUID you targeted);
}

-(void)nearestBeaconChangeTo:(CLBeacon*)beacon
{
    
    _nearistBeaconLabel.text = [NSString stringWithFormat:@"Major:%d Minor:%d",beacon.major.intValue,beacon.minor.intValue];
    
    USBeaconDevice * deviceData = [_manager deviceWithMajor:beacon.major.intValue Minor:beacon.minor.intValue];
    _beaconDataTextView.text = [NSString stringWithFormat:@"名稱: %@\n描述: %@\n電話: %@\n備註: %@\n網址: %@\n自定Key: %@",
                                deviceData.name,
                                deviceData.info.describe,
                                deviceData.info.tel,
                                deviceData.info.note,
                                deviceData.info.url,
                                deviceData.info.extend
                                ];
    
}

//when BeaconDetect detected a beacon power data, it will call this method to notify delegate
-(void)didReceivedPowerVoltage:(float)voltage fromBeacon:(NSString *)mac
{
    //update UI component
    self.BeaconPowerLabel.text = [NSString stringWithFormat:@"Voltage: %1.1f   From_Mac:%@", voltage, mac];
    //do anything you want, upload data to backend server...
}

@end
